### Setup

nscenario <- 5
ndesign <- 6
algorithm <- rep(c("ewoc", "crm", "crm", "wi", "ewoc", "crm"), nscenario)
patients <- rep(c("2p", "2p", "3p", "2p", "2p", "2p"), nscenario)
prior <- rep(c("ni", "ni", "ni", "ni", "i", "i"), nscenario)
scenarios <- rep(c("sa", "sb", "sc", "sd", "se"), each = ndesign)

scenarios_names <- paste(algorithm, patients, sep = "_")
scenarios_names <- paste0(scenarios_names, prior, scenarios)
file <- paste0("results_", scenarios_names, ".Rdata")

p_selection <- matrix(NA, ncol = 5, nrow = length(file))
safety <- matrix(NA, ncol = 3, nrow = length(file))
gp <- list()

### Calculating

for (f in 1:length(file)) {
  file <- paste0("results_", scenarios_names, ".Rdata")
  load(paste0("M:/DinizMA/Research/Drug Combination/CRM/results/discrete/",
              file[f]))
  
  ### Auxiliar functions
  
  pdlt_logistic <- function(rho00, rho01, rho10, eta, theta, x, y){
    
    alpha <- log(rho00/(1 - rho00))
    beta <- log(rho10/(1 - rho10)) - log(rho00/(1 - rho00))
    gamma <- log(rho01/(1 - rho01)) - log(rho00/(1 - rho00))
    out <- plogis(alpha + beta*x + gamma*y + eta*x*y)
    
    return(out)
  }
  
  mtd_logistic <- function(rho00, rho01, rho10, eta, theta, x){
    
    alpha <- log(rho00/(1 - rho00))
    beta <- log(rho10/(1 - rho10)) - log(rho00/(1 - rho00))
    gamma <- log(rho01/(1 - rho01)) - log(rho00/(1 - rho00))
    out <- (log(theta/(1 - theta))- alpha - beta*x)/(gamma + eta*x)
    
    return(out)
  }
  
  mtd_doses <- function(rho00, rho01, rho10, eta, theta, nx, ny, grid) {
    
    # special senarios
    # do not recommend MTD if rho00 > theta + 0.1
    if(rho00 > theta + 0.1) {
      doses = data.frame(x = NA, y = NA)
    } else if(mtd_logistic(rho00, rho01, rho10, eta, theta, 0) < 0) {
      doses = data.frame(x = 0, y = 0)
    } else if(mtd_logistic(rho00, rho01, rho10, eta, theta, 1) > 1) {
      doses = data.frame(x = 1, y = 1)
    } else {
      
      # set number of descrete levels for drug X and Y
      xgrid = unique(grid$x)
      ygrid = unique(grid$y)
      
      # find intercept
      xmin = 0
      xmax = 1
      
      aux_xmax <- function(x, rho00, rho01, rho10, eta, theta) {
        out <- mtd_logistic(rho00, rho01, rho10, eta, theta, x)
      }
      
      aux_xmin <- function(x, rho00, rho01, rho10, eta, theta) {
        out <- mtd_logistic(rho00, rho01, rho10, eta, theta, x) - 1
      }
      
      if(mtd_logistic(rho00,rho01,rho10,eta,theta, 0) > 1) 
        xmin = uniroot(aux_xmin, rho00 = rho00, rho01 = rho01, 
                       rho10 = rho10, eta = eta, 
                       theta = theta, 
                       interval = c(0, 1), extendInt="downX")$root
      if(mtd_logistic(rho00,rho01,rho10,eta,theta, 1) < 0) 
        xmax = uniroot(aux_xmax, rho00 = rho00, rho01 = rho01, 
                       rho10 = rho10, eta = eta, 
                       theta = theta, 
                       interval = c(0,1), extendInt="downX")$root
      
      # Define the function to be minimized
      fdist <- function(x, pt, rho00, rho01, rho10, eta, theta){
        # pt is the point
        d <- (pt[1] - x)^2 + 
          (pt[2]- mtd_logistic(rho00, rho01, rho10, eta, theta, x))^2
        
        out <- d^0.5
        return(out)
      }
      
      ######
      # calculate the mimimum distance for each dose combinations
      ld = matrix(nrow = length(xgrid), ncol = length(ygrid))
      
      for (i in 1:length(xgrid)) { 
        for (j in 1:length(ygrid)) {
          aal <- optimize(f=fdist, c(xmin,xmax), tol=0.0001, c(xgrid[i],ygrid[j]), 
                          rho00, rho01, rho10, eta, theta)
          ld[i,j] <- aal$objective
        }
      }
      
      ######
      # find the best combinations
      
      # 1. select x fist
      ldx <- ld
      for (i in 1:length(xgrid)) 
        ldx[i,which(ldx[i,] != min(ldx[i,], na.rm=T))] <- NA
      
      for (j in 1:length(ygrid)) {
        if (sum(!is.na(ldx[,j])) > 0)
          ldx[which(ldx[,j] != min(ldx[,j], na.rm=T)), j] <- NA
      }
      
      # 2. select y fist
      ldy <- ld
      for (j in 1:length(ygrid)) 
        ldy[which(ldy[,j] != min(ldy[,j], na.rm=T)), j] <- NA
      
      for (i in 1:length(xgrid)) {
        if (sum(!is.na(ldy[i,])) > 0)
          ldy[i,which(ldy[i,] != min(ldy[i,], na.rm=T))] <- NA
      }
      
      # intersection ld1 and ld2
      temp <- data.frame(ldx = as.numeric(ldx), ldy = as.numeric(ldy))
      ld <- matrix(apply(temp[, 1:2], 1, mean), nrow = nx, ncol = ny) # intersection
      ld[is.nan(ld)] <-  NA
      
      # determine the dose combinations for MTD
      dosex <- matrix(xgrid, nrow = length(xgrid), ncol = length(ygrid))
      dosey <- matrix(ygrid, byrow = T, nrow = length(xgrid), ncol = length(ygrid))
      doses <- data.frame(x = dosex[which(!is.na(ld))], y = dosey[which(!is.na(ld))])
      
      #doses is the dose combinations that are close to the true MTD curve
      return(doses)
    }
  }
  
  mtd_selection <- function(rho00_sim, rho01_sim, rho10_sim, eta_sim, theta, 
                            postdlts, postlow, grid, epsilon, 
                            stop = 0.8, mtd_limit = 0.1,
                            file = "mtd_selection") {
    
    mtd_subset <- function(mtd){
      out <- ifelse(all(mtd == 1), 1, 0)
      return(out)
    }
    
    mtd_length <- function(mtd){
      out <- sum(mtd)
      return(out)
    }
    
    mtd_proportion <- function(mtd){
      out <- sum(mtd)/length(mtd)
      return(out)
    }
    
    nx <- nlevels(as.factor(grid$x))
    ny <- nlevels(as.factor(grid$y))
    xgrid = unique(grid$x)
    ygrid = unique(grid$y)
    M <- length(rho00_sim)
    
    # recommend dose combination based on estimated MTD curve
    
    result = data.frame(x=NA, y=NA, trial=NA)
    for (i in 1:M) {
      
      recommended_mtd <- mtd_doses(rho00 = rho00_sim[i], rho01 = rho01_sim[i], 
                                   rho10 = rho10_sim[i], eta = eta_sim[i], 
                                   theta = theta, nx = nx, ny = ny, 
                                   grid = grid)
      
      #################
      # postdlt control, added 03/06/2015
      temp <- subset(as.data.frame(postdlts), trial==i)
      aux <- left_join(recommended_mtd, temp, by = c("x" = "xx", "y" = "yy"))
      
      # If postdlt > delta5, exclude those dose combinations
      aux <- subset(aux, postdlt < epsilon, select = c(x, y))
      ################
      
      if(nrow(aux) > 0) aux$trial = i
      
      # do not recommend MTD if any postlow >= 0.8
      if (max(postlow[i,]) < stop) 
        result <- rbind(result, aux)
    }
    
    # remove NAs (including NA in the first row)
    result = result[!is.na(result$x),]
    
    ######################################################################
    # Percent of MTD individual selection
    ######################################################################
    
    individual <- data.frame(x = grid$x, 
                             y = grid$y, 
                             percent = NA)
    
    for (i in 1:nx) {
      for (j in 1:ny) {
        individual[individual$x == xgrid[i] & individual$y == ygrid[j], 3] <-
          sum(result$x == xgrid[i] & result$y == ygrid[j])*100/M
      }
    }
    
    grid$mtd <- as.factor(ifelse(abs(grid$p - round(theta, 2)) <= mtd_limit, 1, 0))
    gp <- ggplot(grid, aes(x = x, y = y, label = individual$percent, colour = mtd)) + 
      geom_text() + theme_bw() + 
      labs(x = "Dose (Drug A)", y = "Dose (Drug B)") +
      scale_x_continuous(breaks = round(unique(grid$x), 2)) +
      scale_y_continuous(breaks = round(unique(grid$y), 2)) +
      scale_colour_manual(values = c("black", "blue")) +
      theme(panel.grid.major.x = element_line(size = 1/nx), 
            panel.grid.major.y = element_line(size = 1/ny),
            legend.position = "none") + ggsave(file = paste0(file, ".tiff"))
    
    
    ######################################################################
    # Percent of MTD subset selection
    ######################################################################
    aux <- left_join(result, grid)
    aux$trial <- as.factor(aux$trial)
    aux$mtd <- as.numeric(aux$mtd) - 1
    
    ps <- tapply(aux$mtd, aux$trial, mtd_subset)
    ps <- 100*sum(ps)/M
    sk <- tapply(aux$mtd, aux$trial, mtd_length)
    skk <- cumsum(rev(100*table(sk)/M))
    skk <- skk[names(skk) %in% c("3", "2", "1")]
    if(length(skk) < 3) skk <- c(rep(NA, 3 - length(skk)), skk)
    av <- tapply(aux$mtd, aux$trial, mtd_proportion)
    av <- 100*sum(av)/M
    avw <- 100*sum(sk)/length(aux$mtd)
    
    out <- list(subset = ps, length = skk, 
                proportion = av, w.proportion = avw, plot = gp)
    return(out)
  }
  
  mtd_selection_v2 <- function(mtd, grid, mtd_limit, theta) {
    
    M <- 4000
    
    mtd_subset <- function(mtd){
      out <- ifelse(all(mtd == 1), 1, 0)
      return(out)
    }
    
    mtd_length <- function(mtd){
      out <- sum(mtd)
      return(out)
    }
    
    mtd_proportion <- function(mtd){
      out <- sum(mtd)/length(mtd)
      return(out)
    }
    
    grid$mtd <- ifelse(abs(grid$p - round(theta, 2)) <= mtd_limit, 1, 0)
    
    aux <- left_join(mtd, grid)
    aux$trial <- as.factor(aux$trial)

    ps <- tapply(aux$mtd, aux$trial, mtd_subset)
    ps <- 100*sum(ps)/M
    sk <- tapply(aux$mtd, aux$trial, mtd_length)
    skk <- cumsum(rev(100*table(sk)/M))
    skk <- skk[names(skk) %in% c("3", "2", "1")]
    if(length(skk) < 3) skk <- c(rep(NA, 3 - length(skk)), skk)
    av <- tapply(aux$mtd, aux$trial, mtd_proportion)
    av <- 100*sum(av)/M
    avw <- 100*sum(sk)/length(aux$mtd)
    
    out <- list(subset = ps, length = skk, 
                proportion = av, w.proportion = avw)
    return(out)
  }
  
  if (algorithm[f] != "wi"){
    if(ncol(postdlts) > 4)
      postdlts <- postdlts[, -1]
    
    colnames(postdlts) <- c("xx", "yy", "postdlt", "trial")
    
    if (scenarios[f] == "se"){
      postdlts[, 1] <- round(postdlts[, 1], 4)
      postdlts[, 2] <- round(postdlts[, 2], 4)
      grid$x <- round(rep(seq(0,1,length.out=nx), each = ny), 4)
      grid$y <- round(rep(seq(0,1,length.out=ny), nx), 4)
    } else {
      postdlts[, 1] <- round(postdlts[, 1], 1)
      postdlts[, 2] <- round(postdlts[, 2], 1)  
      grid$x <- round(grid$x, 1)
      grid$y <- round(grid$y, 1)
    }
    
    result <- mtd_selection(rho00_sim = vrho00, rho01_sim = vrho01,
                            rho10_sim = vrho10, eta_sim = veta, theta = theta, 
                            postdlts = postdlts, postlow = postlow, 
                            grid = grid, epsilon = 0.65, stop = 0.8, 
                            mtd_limit = 0.10,
                            file =  paste(f, scenarios_names[f], 
                                          "mtd_selection", sep = "_"))
    
    p_selection <- data.frame(subset = round(result$subset, 1), 
                              s3 = round(result$length[1], 1),
                              s2 = round(result$length[2], 1),
                              s1 = round(result$length[3], 1),
                              proportion = round(result$proportion, 1),
                              w.proportion = round(result$w.proportion, 1))
    
    safety <- data.frame(dlt = round(100*mean(rowMeans(vdlt)), 2),
                         dltg5 = round(100*mean(rowMeans(vdlt) > theta + 0.05), 2),
                         dltg10 = round(100*mean(rowMeans(vdlt) > theta + 0.1), 2))
    
    gp[[f]] <- result$plot
  } else {
    result <- mtd_selection_v2(mtd = vmtd, 
                               grid = grid, 
                               mtd_limit = 0.10,
                               theta = theta)
    p_selection <- data.frame(subset = round(result$subset, 1), 
                              s3 = round(result$length[1], 1),
                              s2 = round(result$length[2], 1),
                              s1 = round(result$length[3], 1),
                              proportion = round(result$proportion, 1),
                              w.proportion = round(result$w.proportion, 1))
    
    safety <- data.frame(dlt = round(100*mean(rowMeans(vdlt)), 2),
                         dltg5 = round(100*mean(rowMeans(vdlt) > theta + 0.05), 2),
                         dltg10 = round(100*mean(rowMeans(vdlt) > theta + 0.1), 2))
  }
  
  temp01 <- data.frame(scenarios = scenarios[f],
                       design = paste0(algorithm[f], patients[f]),
                       prior = prior[f], p_selection)
  
  temp02 <- data.frame(scenarios = scenarios[f],
                       design = paste0(algorithm[f], patients[f]),
                       prior = prior[f], safety)
  
  if (f > 1){
    temp01 <- rbind(data_efficiency, temp01)
    temp02 <- rbind(data_safety, temp02)
  }
  
  data_efficiency <- temp01
  data_safety <- temp02
  
  remove(vdlt)
  remove(vrho00)
  remove(vrho01)
  remove(vrho10)
}